// Avisynth v3.0 alpha.  Copyright 2005 David Pierre - Ben Rudiak-Gould et al.
// http://www.avisynth.org

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA, or visit
// http://www.gnu.org/copyleft/gpl.html .
//
// Linking Avisynth statically or dynamically with other modules is making a
// combined work based on Avisynth.  Thus, the terms and conditions of the GNU
// General Public License cover the whole combination.


#ifndef __AVS_GSTREAMER_ELEMENT_H__
#define __AVS_GSTREAMER_ELEMENT_H__

//avisynth include
#include "forward.h"             //for Pad, Object, Pipeline declarations
#include "../core/integer.h"     //for int64

//gstreamer include
#include <gst/gstelement.h>
#include <gst/gstutils.h>        //for gst_element_link


namespace avs { namespace gstreamer {


 
class Element : public GstElement
{

private:  //declared but not implemented

  Element();
  ~Element();


public:  //access to pads

  Pad * GetPad(char const * name);


public:  //link

  bool Link(Element& target) { return gst_element_link(this, &target) != 0; }


public:  //seek method

  bool Seek(uint64 time) { g_message ("time (seek) : %lld", time);
    return gst_element_seek( this,
                                                    1.0,
                                                    GST_FORMAT_TIME,
                                                    (GstSeekFlags)(GST_SEEK_FLAG_ACCURATE | GST_SEEK_FLAG_FLUSH),
                                                    GST_SEEK_TYPE_CUR,
                                                    time,
                                                    GST_SEEK_TYPE_NONE,
                                                    -1) != 0; }


public:  //state stuff

  void SetStateNull() { ChangeState( GST_STATE_NULL ); }
  void SetStateReady() { ChangeState( GST_STATE_READY ); }
  void SetStatePaused() { ChangeState( GST_STATE_PAUSED ); }
  void SetStatePlaying() { ChangeState( GST_STATE_PLAYING ); }
  void SetSimpleStatePaused() { gst_element_set_state( this, GST_STATE_PAUSED ); }


public:

  int64 QueryTotal();


public:  //casts

  operator Object&();

  Pipeline& AsPipeline();


public:  //factory method

  static Element * Create(char const * type, char const * name);


private:  //factory method

  void ChangeState(GstState state);

};



} } //namespace avs::gstreamer

#endif //__AVS_GSTREAMER_ELEMENT_H__
